[systemd-commits] 3 commits - README src/dbus-execute.c src/dbus-execute.h src/execute.c src/execute.h src/load-fragment.c src/main.c src/systemctl.c src/util.c src/util.h TODO

Lennart Poettering lennart at kemper.freedesktop.org
Thu Mar 3 20:10:24 PST 2011


 README              |    5 +
 TODO                |    5 -
 src/dbus-execute.c  |   33 +++++++++
 src/dbus-execute.h  |    2 
 src/execute.c       |  184 +++++++++++++++++++++++++++++++++++++---------------
 src/execute.h       |    4 +
 src/load-fragment.c |   70 ++-----------------
 src/main.c          |   13 +--
 src/systemctl.c     |   19 +++++
 src/util.c          |   67 ++++++++++++++++++
 src/util.h          |    1 
 11 files changed, 279 insertions(+), 124 deletions(-)

New commits:
commit aa1671320df5bd8aa4108c0efb34a49fdedd0e5f
Author: Lennart Poettering <lennart at poettering.net>
Date:   Fri Mar 4 05:07:01 2011 +0100

    main: add link to wiki page with longer explanation of the /usr madness

diff --git a/README b/README
index a1d100c..96fdc5f 100644
--- a/README
+++ b/README
@@ -72,6 +72,9 @@ WARNINGS:
         about this, since this kind of file system setup is not really
         supported anymore by the basic set of Linux OS components.
 
+        For more information on this issue consult
+        http://freedesktop.org/wiki/Software/systemd/separate-usr-is-broken
+
 ENGINEERING AND CONSULTING SERVICES:
         ProFUSION <http://profusion.mobi> offers professional
         engineering and consulting services for systemd for embedded
diff --git a/src/main.c b/src/main.c
index cf54c7a..3291561 100644
--- a/src/main.c
+++ b/src/main.c
@@ -961,7 +961,8 @@ static void test_usr(void) {
                 return;
 
         log_warning("/usr appears to be on a different file system than /. This is not supported anymore. "
-                    "Some things will probably break (sometimes even silently) in mysterious ways.");
+                    "Some things will probably break (sometimes even silently) in mysterious ways. "
+                    "Consult http://freedesktop.org/wiki/Software/systemd/separate-usr-is-broken for more information.");
 }
 
 int main(int argc, char *argv[]) {

commit ed1c99fcc13f279eccf25fc20427eb92867da245
Author: Lennart Poettering <lennart at poettering.net>
Date:   Fri Mar 4 03:58:52 2011 +0100

    s/seperate/separate/

diff --git a/README b/README
index aed2715..a1d100c 100644
--- a/README
+++ b/README
@@ -63,7 +63,7 @@ WARNINGS:
 
         systemd will warn you during boot if /usr is on a different
         file system than /. While in systemd itself very little will
-        break if /usr is on a seperate partition many of its
+        break if /usr is on a separate partition many of its
         dependencies very likely will break sooner or later in one
         form or another. For example udev rules tend to refer to
         binaries in /usr, binaries that link to libraries in /usr or
diff --git a/TODO b/TODO
index ecf9c05..14060a5 100644
--- a/TODO
+++ b/TODO
@@ -14,8 +14,6 @@ F15:
 
 * hook emergency.target into local-fs.target in some way as OnFailure with isolate
 
-* s/seperate/separate/
-
 Features:
 
 * introduce "x-systemd-automount" as alternative to the "comment=systemd.automount" mount option
@@ -117,7 +115,7 @@ Features:
 
 * readahead: btrfs/LVM SSD detection
 
-* add seperate man page for [Install] settings
+* add separate man page for [Install] settings
 
 * only add quotacheck deps to .mount units which mention grpquota/usrquota in the mount flags
 
diff --git a/TODO b/TODO
index 36c738e..ecf9c05 100644
--- a/TODO
+++ b/TODO
@@ -8,15 +8,14 @@ F15:
 
 * finish syslog socket stuff
 
-* load EnvironmentFile= when starting services, not when reloading configuration
-  https://bugzilla.redhat.com/show_bug.cgi?id=661282
-
 * NFS, networkmanager ordering issue
 
 * add fstab fields to add wait timeouts, change Wants to Requires by local-fs.target
 
 * hook emergency.target into local-fs.target in some way as OnFailure with isolate
 
+* s/seperate/separate/
+
 Features:
 
 * introduce "x-systemd-automount" as alternative to the "comment=systemd.automount" mount option
diff --git a/src/dbus-execute.c b/src/dbus-execute.c
index cdc15b2..504651f 100644
--- a/src/dbus-execute.c
+++ b/src/dbus-execute.c
@@ -33,6 +33,39 @@ DEFINE_BUS_PROPERTY_APPEND_ENUM(bus_execute_append_kill_mode, kill_mode, KillMod
 DEFINE_BUS_PROPERTY_APPEND_ENUM(bus_execute_append_input, exec_input, ExecInput);
 DEFINE_BUS_PROPERTY_APPEND_ENUM(bus_execute_append_output, exec_output, ExecOutput);
 
+int bus_execute_append_env_files(Manager *m, DBusMessageIter *i, const char *property, void *data) {
+        char **env_files = data, **j;
+        DBusMessageIter sub, sub2;
+
+        assert(m);
+        assert(i);
+        assert(property);
+
+        if (!dbus_message_iter_open_container(i, DBUS_TYPE_ARRAY, "(sb)", &sub))
+                return -ENOMEM;
+
+        STRV_FOREACH(j, env_files) {
+                dbus_bool_t b = false;
+                char *fn = *j;
+
+                if (fn[0] == '-') {
+                        b = true;
+                        fn++;
+                }
+
+                if (!dbus_message_iter_open_container(&sub, DBUS_TYPE_STRUCT, NULL, &sub2) ||
+                    !dbus_message_iter_append_basic(&sub2, DBUS_TYPE_STRING, &fn) ||
+                    !dbus_message_iter_append_basic(&sub2, DBUS_TYPE_BOOLEAN, &b) ||
+                    !dbus_message_iter_close_container(&sub, &sub2))
+                        return -ENOMEM;
+        }
+
+        if (!dbus_message_iter_close_container(i, &sub))
+                return -ENOMEM;
+
+        return 0;
+}
+
 int bus_execute_append_oom_score_adjust(Manager *m, DBusMessageIter *i, const char *property, void *data) {
         ExecContext *c = data;
         int32_t n;
diff --git a/src/dbus-execute.h b/src/dbus-execute.h
index 87dc698..082456a 100644
--- a/src/dbus-execute.h
+++ b/src/dbus-execute.h
@@ -93,6 +93,7 @@
 
 #define BUS_EXEC_CONTEXT_PROPERTIES(interface, context)                 \
         { interface, "Environment",                   bus_property_append_strv,   "as",    (context).environment                   }, \
+        { interface, "EnvironmentFiles",              bus_execute_append_env_files, "a(sb)", (context).environment_files           }, \
         { interface, "UMask",                         bus_property_append_mode,   "u",     &(context).umask                        }, \
         { interface, "LimitCPU",                      bus_execute_append_rlimits, "t",     &(context)                              }, \
         { interface, "LimitFSIZE",                    bus_execute_append_rlimits, "t",     &(context)                              }, \
@@ -169,5 +170,6 @@ int bus_execute_append_capabilities(Manager *m, DBusMessageIter *i, const char *
 int bus_execute_append_rlimits(Manager *m, DBusMessageIter *i, const char *property, void *data);
 int bus_execute_append_command(Manager *m, DBusMessageIter *u, const char *property, void *data);
 int bus_execute_append_kill_mode(Manager *m, DBusMessageIter *i, const char *property, void *data);
+int bus_execute_append_env_files(Manager *m, DBusMessageIter *i, const char *property, void *data);
 
 #endif
diff --git a/src/execute.c b/src/execute.c
index 281d90d..ee05e99 100644
--- a/src/execute.c
+++ b/src/execute.c
@@ -919,6 +919,7 @@ int exec_spawn(ExecCommand *command,
         int r;
         char *line;
         int socket_fd;
+        char **files_env = NULL;
 
         assert(command);
         assert(context);
@@ -939,21 +940,30 @@ int exec_spawn(ExecCommand *command,
         } else
                 socket_fd = -1;
 
+        if ((r = exec_context_load_environment(context, &files_env)) < 0) {
+                log_error("Failed to load environment files: %s", strerror(-r));
+                return r;
+        }
+
         if (!argv)
                 argv = command->argv;
 
-        if (!(line = exec_command_line(argv)))
-                return -ENOMEM;
+        if (!(line = exec_command_line(argv))) {
+                r = -ENOMEM;
+                goto fail_parent;
+        }
 
         log_debug("About to execute: %s", line);
         free(line);
 
         if (cgroup_bondings)
                 if ((r = cgroup_bonding_realize_list(cgroup_bondings)))
-                        return r;
+                        goto fail_parent;
 
-        if ((pid = fork()) < 0)
-                return -errno;
+        if ((pid = fork()) < 0) {
+                r = -errno;
+                goto fail_parent;
+        }
 
         if (pid == 0) {
                 int i;
@@ -983,7 +993,7 @@ int exec_spawn(ExecCommand *command,
                 if (sigemptyset(&ss) < 0 ||
                     sigprocmask(SIG_SETMASK, &ss, NULL) < 0) {
                         r = EXIT_SIGNAL_MASK;
-                        goto fail;
+                        goto fail_child;
                 }
 
                 /* Close sockets very early to make sure we don't
@@ -992,26 +1002,26 @@ int exec_spawn(ExecCommand *command,
                 if (close_all_fds(socket_fd >= 0 ? &socket_fd : fds,
                                   socket_fd >= 0 ? 1 : n_fds) < 0) {
                         r = EXIT_FDS;
-                        goto fail;
+                        goto fail_child;
                 }
 
                 if (!context->same_pgrp)
                         if (setsid() < 0) {
                                 r = EXIT_SETSID;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 if (context->tcpwrap_name) {
                         if (socket_fd >= 0)
                                 if (!socket_tcpwrap(socket_fd, context->tcpwrap_name)) {
                                         r = EXIT_TCPWRAP;
-                                        goto fail;
+                                        goto fail_child;
                                 }
 
                         for (i = 0; i < (int) n_fds; i++) {
                                 if (!socket_tcpwrap(fds[i], context->tcpwrap_name)) {
                                         r = EXIT_TCPWRAP;
-                                        goto fail;
+                                        goto fail_child;
                                 }
                         }
                 }
@@ -1024,12 +1034,12 @@ int exec_spawn(ExecCommand *command,
                         /* Set up terminal for the question */
                         if ((r = setup_confirm_stdio(context,
                                                      &saved_stdin, &saved_stdout)))
-                                goto fail;
+                                goto fail_child;
 
                         /* Now ask the question. */
                         if (!(line = exec_command_line(argv))) {
                                 r = EXIT_MEMORY;
-                                goto fail;
+                                goto fail_child;
                         }
 
                         r = ask(&response, "yns", "Execute %s? [Yes, No, Skip] ", line);
@@ -1037,17 +1047,17 @@ int exec_spawn(ExecCommand *command,
 
                         if (r < 0 || response == 'n') {
                                 r = EXIT_CONFIRM;
-                                goto fail;
+                                goto fail_child;
                         } else if (response == 's') {
                                 r = 0;
-                                goto fail;
+                                goto fail_child;
                         }
 
                         /* Release terminal for the question */
                         if ((r = restore_confirm_stdio(context,
                                                        &saved_stdin, &saved_stdout,
                                                        &keep_stdin, &keep_stdout)))
-                                goto fail;
+                                goto fail_child;
                 }
 
                 /* If a socket is connected to STDIN/STDOUT/STDERR, we
@@ -1058,24 +1068,24 @@ int exec_spawn(ExecCommand *command,
                 if (!keep_stdin)
                         if (setup_input(context, socket_fd, apply_tty_stdin) < 0) {
                                 r = EXIT_STDIN;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 if (!keep_stdout)
                         if (setup_output(context, socket_fd, file_name_from_path(command->path), apply_tty_stdin) < 0) {
                                 r = EXIT_STDOUT;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 if (setup_error(context, socket_fd, file_name_from_path(command->path), apply_tty_stdin) < 0) {
                         r = EXIT_STDERR;
-                        goto fail;
+                        goto fail_child;
                 }
 
                 if (cgroup_bondings)
                         if (cgroup_bonding_install_list(cgroup_bondings, 0) < 0) {
                                 r = EXIT_CGROUP;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 if (context->oom_score_adjust_set) {
@@ -1097,7 +1107,7 @@ int exec_spawn(ExecCommand *command,
 
                                 if (write_one_line_file("/proc/self/oom_adj", t) < 0) {
                                         r = EXIT_OOM_ADJUST;
-                                        goto fail;
+                                        goto fail_child;
                                 }
                         }
                 }
@@ -1105,7 +1115,7 @@ int exec_spawn(ExecCommand *command,
                 if (context->nice_set)
                         if (setpriority(PRIO_PROCESS, 0, context->nice) < 0) {
                                 r = EXIT_NICE;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 if (context->cpu_sched_set) {
@@ -1117,26 +1127,26 @@ int exec_spawn(ExecCommand *command,
                         if (sched_setscheduler(0, context->cpu_sched_policy |
                                                (context->cpu_sched_reset_on_fork ? SCHED_RESET_ON_FORK : 0), &param) < 0) {
                                 r = EXIT_SETSCHEDULER;
-                                goto fail;
+                                goto fail_child;
                         }
                 }
 
                 if (context->cpuset)
                         if (sched_setaffinity(0, CPU_ALLOC_SIZE(context->cpuset_ncpus), context->cpuset) < 0) {
                                 r = EXIT_CPUAFFINITY;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 if (context->ioprio_set)
                         if (ioprio_set(IOPRIO_WHO_PROCESS, 0, context->ioprio) < 0) {
                                 r = EXIT_IOPRIO;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 if (context->timer_slack_nsec_set)
                         if (prctl(PR_SET_TIMERSLACK, context->timer_slack_nsec) < 0) {
                                 r = EXIT_TIMERSLACK;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 if (context->utmp_id)
@@ -1146,13 +1156,13 @@ int exec_spawn(ExecCommand *command,
                         username = context->user;
                         if (get_user_creds(&username, &uid, &gid, &home) < 0) {
                                 r = EXIT_USER;
-                                goto fail;
+                                goto fail_child;
                         }
 
                         if (is_terminal_input(context->std_input))
                                 if (chown_terminal(STDIN_FILENO, uid) < 0) {
                                         r = EXIT_STDIN;
-                                        goto fail;
+                                        goto fail_child;
                                 }
                 }
 
@@ -1160,7 +1170,7 @@ int exec_spawn(ExecCommand *command,
                 if (context->pam_name && username) {
                         if (setup_pam(context->pam_name, username, context->tty_path, &pam_env, fds, n_fds) < 0) {
                                 r = EXIT_PAM;
-                                goto fail;
+                                goto fail_child;
                         }
                 }
 #endif
@@ -1168,7 +1178,7 @@ int exec_spawn(ExecCommand *command,
                 if (apply_permissions)
                         if (enforce_groups(context, username, uid) < 0) {
                                 r = EXIT_GROUP;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 umask(context->umask);
@@ -1184,18 +1194,18 @@ int exec_spawn(ExecCommand *command,
                                              context->inaccessible_dirs,
                                              context->private_tmp,
                                              context->mount_flags)) < 0)
-                                goto fail;
+                                goto fail_child;
 
                 if (apply_chroot) {
                         if (context->root_directory)
                                 if (chroot(context->root_directory) < 0) {
                                         r = EXIT_CHROOT;
-                                        goto fail;
+                                        goto fail_child;
                                 }
 
                         if (chdir(context->working_directory ? context->working_directory : "/") < 0) {
                                 r = EXIT_CHDIR;
-                                goto fail;
+                                goto fail_child;
                         }
                 } else {
 
@@ -1205,13 +1215,13 @@ int exec_spawn(ExecCommand *command,
                                      context->root_directory ? context->root_directory : "",
                                      context->working_directory ? context->working_directory : "") < 0) {
                                 r = EXIT_MEMORY;
-                                goto fail;
+                                goto fail_child;
                         }
 
                         if (chdir(d) < 0) {
                                 free(d);
                                 r = EXIT_CHDIR;
-                                goto fail;
+                                goto fail_child;
                         }
 
                         free(d);
@@ -1223,7 +1233,7 @@ int exec_spawn(ExecCommand *command,
                     shift_fds(fds, n_fds) < 0 ||
                     flags_fds(fds, n_fds, context->non_blocking) < 0) {
                         r = EXIT_FDS;
-                        goto fail;
+                        goto fail_child;
                 }
 
                 if (apply_permissions) {
@@ -1234,14 +1244,14 @@ int exec_spawn(ExecCommand *command,
 
                                 if (setrlimit(i, context->rlimit[i]) < 0) {
                                         r = EXIT_LIMITS;
-                                        goto fail;
+                                        goto fail_child;
                                 }
                         }
 
                         if (context->user)
                                 if (enforce_user(context, uid) < 0) {
                                         r = EXIT_USER;
-                                        goto fail;
+                                        goto fail_child;
                                 }
 
                         /* PR_GET_SECUREBITS is not privileged, while
@@ -1251,39 +1261,39 @@ int exec_spawn(ExecCommand *command,
                         if (prctl(PR_GET_SECUREBITS) != context->secure_bits)
                                 if (prctl(PR_SET_SECUREBITS, context->secure_bits) < 0) {
                                         r = EXIT_SECUREBITS;
-                                        goto fail;
+                                        goto fail_child;
                                 }
 
                         if (context->capabilities)
                                 if (cap_set_proc(context->capabilities) < 0) {
                                         r = EXIT_CAPABILITIES;
-                                        goto fail;
+                                        goto fail_child;
                                 }
                 }
 
                 if (!(our_env = new0(char*, 7))) {
                         r = EXIT_MEMORY;
-                        goto fail;
+                        goto fail_child;
                 }
 
                 if (n_fds > 0)
                         if (asprintf(our_env + n_env++, "LISTEN_PID=%lu", (unsigned long) getpid()) < 0 ||
                             asprintf(our_env + n_env++, "LISTEN_FDS=%u", n_fds) < 0) {
                                 r = EXIT_MEMORY;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 if (home)
                         if (asprintf(our_env + n_env++, "HOME=%s", home) < 0) {
                                 r = EXIT_MEMORY;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 if (username)
                         if (asprintf(our_env + n_env++, "LOGNAME=%s", username) < 0 ||
                             asprintf(our_env + n_env++, "USER=%s", username) < 0) {
                                 r = EXIT_MEMORY;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 if (is_terminal_input(context->std_input) ||
@@ -1291,25 +1301,26 @@ int exec_spawn(ExecCommand *command,
                     context->std_error == EXEC_OUTPUT_TTY)
                         if (!(our_env[n_env++] = strdup(default_term_for_tty(tty_path(context))))) {
                                 r = EXIT_MEMORY;
-                                goto fail;
+                                goto fail_child;
                         }
 
                 assert(n_env <= 7);
 
                 if (!(final_env = strv_env_merge(
-                                      4,
+                                      5,
                                       environment,
                                       our_env,
                                       context->environment,
+                                      files_env,
                                       pam_env,
                                       NULL))) {
                         r = EXIT_MEMORY;
-                        goto fail;
+                        goto fail_child;
                 }
 
                 if (!(final_argv = replace_env_argv(argv, final_env))) {
                         r = EXIT_MEMORY;
-                        goto fail;
+                        goto fail_child;
                 }
 
                 final_env = strv_env_clean(final_env);
@@ -1317,10 +1328,11 @@ int exec_spawn(ExecCommand *command,
                 execve(command->path, final_argv, final_env);
                 r = EXIT_EXEC;
 
-        fail:
+        fail_child:
                 strv_free(our_env);
                 strv_free(final_env);
                 strv_free(pam_env);
+                strv_free(files_env);
                 strv_free(final_argv);
 
                 if (saved_stdin >= 0)
@@ -1332,6 +1344,8 @@ int exec_spawn(ExecCommand *command,
                 _exit(r);
         }
 
+        strv_free(files_env);
+
         /* We add the new process to the cgroup both in the child (so
          * that we can be sure that no user code is ever executed
          * outside of the cgroup) and in the parent (so that we can be
@@ -1346,6 +1360,11 @@ int exec_spawn(ExecCommand *command,
 
         *ret = pid;
         return 0;
+
+fail_parent:
+        strv_free(files_env);
+
+        return r;
 }
 
 void exec_context_init(ExecContext *c) {
@@ -1369,6 +1388,9 @@ void exec_context_done(ExecContext *c) {
         strv_free(c->environment);
         c->environment = NULL;
 
+        strv_free(c->environment_files);
+        c->environment_files = NULL;
+
         for (l = 0; l < ELEMENTSOF(c->rlimit); l++) {
                 free(c->rlimit[l]);
                 c->rlimit[l] = NULL;
@@ -1457,6 +1479,64 @@ void exec_command_free_array(ExecCommand **c, unsigned n) {
         }
 }
 
+int exec_context_load_environment(const ExecContext *c, char ***l) {
+        char **i, **r = NULL;
+
+        assert(c);
+        assert(l);
+
+        STRV_FOREACH(i, c->environment_files) {
+                char *fn;
+                int k;
+                bool ignore = false;
+                char **p;
+
+                fn = *i;
+
+                if (fn[0] == '-') {
+                        ignore = true;
+                        fn ++;
+                }
+
+                if (!path_is_absolute(fn)) {
+
+                        if (ignore)
+                                continue;
+
+                        strv_free(r);
+                        return -EINVAL;
+                }
+
+                if ((k = load_env_file(fn, &p)) < 0) {
+
+                        if (ignore)
+                                continue;
+
+                        strv_free(r);
+                        return k;
+                }
+
+                if (r == NULL)
+                        r = p;
+                else {
+                        char **m;
+
+                        m = strv_env_merge(2, r, p);
+                        strv_free(r);
+                        strv_free(p);
+
+                        if (!m)
+                                return -ENOMEM;
+
+                        r = m;
+                }
+        }
+
+        *l = r;
+
+        return 0;
+}
+
 static void strv_fprintf(FILE *f, char **l) {
         char **g;
 
@@ -1488,9 +1568,11 @@ void exec_context_dump(ExecContext *c, FILE* f, const char *prefix) {
                 prefix, yes_no(c->non_blocking),
                 prefix, yes_no(c->private_tmp));
 
-        if (c->environment)
-                for (e = c->environment; *e; e++)
-                        fprintf(f, "%sEnvironment: %s\n", prefix, *e);
+        STRV_FOREACH(e, c->environment)
+                fprintf(f, "%sEnvironment: %s\n", prefix, *e);
+
+        STRV_FOREACH(e, c->environment_files)
+                fprintf(f, "%sEnvironmentFile: %s\n", prefix, *e);
 
         if (c->tcpwrap_name)
                 fprintf(f,
diff --git a/src/execute.h b/src/execute.h
index 3cdd2ad..2856d2f 100644
--- a/src/execute.h
+++ b/src/execute.h
@@ -104,6 +104,8 @@ struct ExecCommand {
 
 struct ExecContext {
         char **environment;
+        char **environment_files;
+
         struct rlimit *rlimit[RLIMIT_NLIMITS];
         char *working_directory, *root_directory;
 
@@ -203,6 +205,8 @@ void exec_context_init(ExecContext *c);
 void exec_context_done(ExecContext *c);
 void exec_context_dump(ExecContext *c, FILE* f, const char *prefix);
 
+int exec_context_load_environment(const ExecContext *c, char ***l);
+
 void exec_status_start(ExecStatus *s, pid_t pid);
 void exec_status_exit(ExecStatus *s, pid_t pid, int code, int status, const char *utmp_id);
 void exec_status_dump(ExecStatus *s, FILE *f, const char *prefix);
diff --git a/src/load-fragment.c b/src/load-fragment.c
index 2e67ecc..bd7529f 100644
--- a/src/load-fragment.c
+++ b/src/load-fragment.c
@@ -1340,79 +1340,25 @@ static int config_parse_env_file(
                 void *data,
                 void *userdata) {
 
-        FILE *f;
-        int r;
-        char ***env = data;
-        bool ignore = false;
+        char ***env = data, **k;
 
         assert(filename);
         assert(lvalue);
         assert(rvalue);
         assert(data);
 
-        if (rvalue[0] == '-') {
-                ignore = true;
-                rvalue++;
-        }
-
-        if (!path_is_absolute(rvalue)) {
+        if (!path_is_absolute(rvalue[0] == '-' ? rvalue + 1 : rvalue)) {
                 log_error("[%s:%u] Path '%s' is not absolute, ignoring.", filename, line, rvalue);
                 return 0;
         }
 
-        if (!(f = fopen(rvalue, "re"))) {
-                if (!ignore)
-                        log_error("[%s:%u] Failed to open environment file '%s', ignoring: %m", filename, line, rvalue);
-                return 0;
-        }
-
-        while (!feof(f)) {
-                char l[LINE_MAX], *p, *u;
-                char **t;
-
-                if (!fgets(l, sizeof(l), f)) {
-                        if (feof(f))
-                                break;
-
-                        log_error("[%s:%u] Failed to read environment file '%s', ignoring: %m", filename, line, rvalue);
-                        r = 0;
-                        goto finish;
-                }
-
-                p = strstrip(l);
-
-                if (!*p)
-                        continue;
-
-                if (strchr(COMMENTS, *p))
-                        continue;
-
-                if (!(u = normalize_env_assignment(p))) {
-                        log_error("Out of memory");
-                        r = -ENOMEM;
-                        goto finish;
-                }
-
-                t = strv_append(*env, u);
-                free(u);
-
-                if (!t) {
-                        log_error("Out of memory");
-                        r = -ENOMEM;
-                        goto finish;
-                }
-
-                strv_free(*env);
-                *env = t;
-        }
-
-        r = 0;
+        if (!(k = strv_append(*env, rvalue)))
+                return -ENOMEM;
 
-finish:
-        if (f)
-                fclose(f);
+        strv_free(*env);
+        *env = k;
 
-        return r;
+        return 0;
 }
 
 static int config_parse_ip_tos(
@@ -1803,7 +1749,7 @@ static int load_from_path(Unit *u, const char *path) {
                 { "CPUAffinity",            config_parse_cpu_affinity,    &(context),                                      section   }, \
                 { "UMask",                  config_parse_mode,            &(context).umask,                                section   }, \
                 { "Environment",            config_parse_strv,            &(context).environment,                          section   }, \
-                { "EnvironmentFile",        config_parse_env_file,        &(context).environment,                          section   }, \
+                { "EnvironmentFile",        config_parse_env_file,        &(context).environment_files,                    section   }, \
                 { "StandardInput",          config_parse_input,           &(context).std_input,                            section   }, \
                 { "StandardOutput",         config_parse_output,          &(context).std_output,                           section   }, \
                 { "StandardError",          config_parse_output,          &(context).std_error,                            section   }, \
diff --git a/src/systemctl.c b/src/systemctl.c
index 2e8d107..b733b47 100644
--- a/src/systemctl.c
+++ b/src/systemctl.c
@@ -2370,6 +2370,25 @@ static int print_property(const char *name, DBusMessageIter *iter) {
 
                         return 0;
 
+                } else if (dbus_message_iter_get_element_type(iter) == DBUS_TYPE_STRUCT && streq(name, "EnvironmentFiles")) {
+                        DBusMessageIter sub, sub2;
+
+                        dbus_message_iter_recurse(iter, &sub);
+                        while (dbus_message_iter_get_arg_type(&sub) == DBUS_TYPE_STRUCT) {
+                                const char *path;
+                                dbus_bool_t ignore;
+
+                                dbus_message_iter_recurse(&sub, &sub2);
+
+                                if (bus_iter_get_basic_and_next(&sub2, DBUS_TYPE_STRING, &path, true) >= 0 &&
+                                    bus_iter_get_basic_and_next(&sub2, DBUS_TYPE_BOOLEAN, &ignore, false) >= 0)
+                                        printf("EnvironmentFile=%s (ignore=%s)\n", path, yes_no(ignore));
+
+                                dbus_message_iter_next(&sub);
+                        }
+
+                        return 0;
+
                 } else if (dbus_message_iter_get_element_type(iter) == DBUS_TYPE_STRUCT && streq(name, "Paths")) {
                         DBusMessageIter sub, sub2;
 
diff --git a/src/util.c b/src/util.c
index 23d7e22..b02a77e 100644
--- a/src/util.c
+++ b/src/util.c
@@ -683,6 +683,73 @@ fail:
         return r;
 }
 
+int load_env_file(
+                const char *fname,
+                char ***rl) {
+
+        FILE *f;
+        char **m = 0;
+        int r;
+
+        assert(fname);
+        assert(rl);
+
+        if (!(f = fopen(fname, "re")))
+                return -errno;
+
+        while (!feof(f)) {
+                char l[LINE_MAX], *p, *u;
+                char **t;
+
+                if (!fgets(l, sizeof(l), f)) {
+                        if (feof(f))
+                                break;
+
+                        r = -errno;
+                        goto finish;
+                }
+
+                p = strstrip(l);
+
+                if (!*p)
+                        continue;
+
+                if (strchr(COMMENTS, *p))
+                        continue;
+
+                if (!(u = normalize_env_assignment(p))) {
+                        log_error("Out of memory");
+                        r = -ENOMEM;
+                        goto finish;
+                }
+
+                t = strv_append(m, u);
+                free(u);
+
+                if (!t) {
+                        log_error("Out of memory");
+                        r = -ENOMEM;
+                        goto finish;
+                }
+
+                strv_free(m);
+                m = t;
+        }
+
+        r = 0;
+
+        *rl = m;
+        m = NULL;
+
+finish:
+        if (f)
+                fclose(f);
+
+        strv_free(m);
+
+        return r;
+}
+
 char *truncate_nl(char *s) {
         assert(s);
 
diff --git a/src/util.h b/src/util.h
index 3f3347d..a2e3b94 100644
--- a/src/util.h
+++ b/src/util.h
@@ -195,6 +195,7 @@ int read_one_line_file(const char *fn, char **line);
 int read_full_file(const char *fn, char **contents);
 
 int parse_env_file(const char *fname, const char *separator, ...) _sentinel_;
+int load_env_file(const char *fname, char ***l);
 
 char *strappend(const char *s, const char *suffix);
 char *strnappend(const char *s, const char *suffix, size_t length);



More information about the systemd-commits mailing list
diff --git a/src/main.c b/src/main.c
index 2c02217..cf54c7a 100644
--- a/src/main.c
+++ b/src/main.c
@@ -942,22 +942,22 @@ static void test_mtab(void) {
 
 static void test_usr(void) {
         struct stat a, b;
-        bool seperate = false;
+        bool separate = false;
 
         /* Check that /usr is not a seperate fs */
 
         if (lstat("/", &a) >= 0 && lstat("/usr", &b) >= 0)
                 if (a.st_dev != b.st_dev)
-                        seperate = true;
+                        separate = true;
 
         /* This check won't work usually during boot, since /usr is
          * probably not mounted yet, hence let's add a second
          * check. We just check whether /usr is an empty directory. */
 
         if (dir_is_empty("/usr") > 0)
-                seperate = true;
+                separate = true;
 
-        if (!seperate)
+        if (!separate)
                 return;
 
         log_warning("/usr appears to be on a different file system than /. This is not supported anymore. "
@@ -942,22 +942,22 @@ static void test_mtab(void) {
 
-        /* Check that /usr is not a seperate fs */
+        /* Check that /usr is not a separate fs */
 
         if (lstat("/", &a) >= 0 && lstat("/usr", &b) >= 0)

commit 8c7be95e5a80c4bd82d86e9640a071fd98618172
Author: Lennart Poettering <lennart at poettering.net>
Date:   Fri Mar 4 03:44:43 2011 +0100

    execute: load environment files at time of execution, not when we load the service configuration
    
    https://bugzilla.redhat.com/show_bug.cgi?id=661282
